// Temp24hrMaxMin.cpp
/*
   (c)2022 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/

#include "Temp24hrMaxMin.h"
#include "millisDelay.h"

Temp24hrMaxMin::Temp24hrMaxMin() {
  // initialize to invalid 1000.0degC
  currentMaxMin.max = invalid;
  currentMaxMin.min = invalid;
  maxMinLast24hrs = currentMaxMin;
  for (size_t i = 0; i < ARRAY_SIZE; i++) {
    maxMinArray[i] = currentMaxMin;
  }
  periodDelay.start(PERIOD_DELAY_MS); // start timer to rotate array
  idx = 0; // next idx to put current when timer times out
}

float Temp24hrMaxMin::getMax() {
  return maxMinLast24hrs.max;
}
float Temp24hrMaxMin::getMin() {
  return maxMinLast24hrs.min;
}

// only first arg checked for invalid
float Temp24hrMaxMin::updateMax(float _f1, float _f2) {
  if ((_f1 >= invalid) || (_f1 < _f2)) {
    return _f2;
  }
  // else
  return _f1;
}

// only first arg checked for invalid
float Temp24hrMaxMin::updateMin(float _f1, float _f2) {
  if ((_f1 >= invalid) || (_f1 > _f2)) {
    return _f2;
  }
  // else
  return _f1;
}

void Temp24hrMaxMin::setDeg(float _fvalue) {
  if (_fvalue >= invalid) {
    // skip
    return;
  }
  // update current and maxMinLast24hrs
  currentMaxMin.max = updateMax(currentMaxMin.max, _fvalue);
  currentMaxMin.min = updateMin(currentMaxMin.min, _fvalue);
  maxMinLast24hrs.max = updateMax(maxMinLast24hrs.max, currentMaxMin.max);
  maxMinLast24hrs.min = updateMin(maxMinLast24hrs.min, currentMaxMin.min);
  if (periodDelay.justFinished()) {
    periodDelay.repeat(); // no slip
    // rotate current into array and recalculate max/min for array
    maxMinArray[idx] = currentMaxMin;
    idx++;
    if (idx >= ARRAY_SIZE) {
      idx = 0; // wrap
    }
    maxMinLast24hrs = currentMaxMin;
    currentMaxMin.max = _fvalue;
    currentMaxMin.min = _fvalue;
    // find new maxMinLast24hrs
    for (size_t i = 0; i < ARRAY_SIZE; i++) {
      maxMinLast24hrs.max = updateMax(maxMinArray[i].max, maxMinLast24hrs.max);
      maxMinLast24hrs.min = updateMin(maxMinArray[i].min, maxMinLast24hrs.min);
    }
  }
}

void Temp24hrMaxMin::printMaxMin(struct MaxMinStruct maxmin, Print* out) {
  if (!out) {
    return;
  }
  out->print("max:"); out->print(maxmin.max); out->print(" min:"); out->print(maxmin.min);
  out->println();
}
