#ifndef Temp24hrMaxMin_h
#define Temp24hrMaxMin_h
//  Temp24hrMaxMin.h
// accumulates max/min over last 24hrs in 15min interval
/*
   (c)2014-2022 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/
#include "Arduino.h"
#include "millisDelay.h"

class Temp24hrMaxMin {
  public:
    Temp24hrMaxMin();
    void setDeg(float _fvalue);
    float getMax();
    float getMin();
    const static int invalid = 1000;

  private:
    struct MaxMinStruct {
      float max; // max degC for this period
      float min; // min degC for this period
    };
    void printMaxMin(struct MaxMinStruct maxmin, Print* out); // for debugging

    struct MaxMinStruct currentMaxMin;
    struct MaxMinStruct maxMinLast24hrs;
    static const size_t ARRAY_SIZE = 96;  // <<<<<<< forces all of these classed to have array size 96
    struct MaxMinStruct maxMinArray[ARRAY_SIZE]; 

    float updateMax(float _f1, float _f2);
    float updateMin(float _f1, float _f2);
    unsigned long ms;
    const unsigned long PERIOD_DELAY_MS = 24ul * 60 * 60 * 1000 / ARRAY_SIZE; // 30mins for ARRAY_SIZE==48, 15mins for ARRAY_SIZE==96;
    millisDelay periodDelay;
    size_t idx; // next idx in maxMinArray to use for current
};
#endif // Temp24hrMaxMin_h
