/*
   (c)2014-2022 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/

#include "rh_Gauge.h"
#include "SafeString.h"
// 20 to 100% => 0 to 180 deg
static float degStart = 190;
static float degEnd = -10;
static float degRange = degStart - degEnd; // arc from left to rugh
static float valueToAngle(float value) {
  float ang = (value-20)*100/80.0 * (degRange/100);
  if (ang > degRange) {
    ang = degRange;
  }
  if (ang < 0) {
    ang = 0;
  }
  return ang;
}

rh_Gauge::rh_Gauge(pfodDwgs *_dwgsPtr) : pfodControl(_dwgsPtr) {
  z_idx = pfodDwgs::reserveIdx(4);
}

void rh_Gauge::drawScalePoint(int rh) {
  cSF(sfLabel, 20);
  sfLabel = rh;
  sfLabel += '%';
  float ang = degStart - valueToAngle(rh);
  dwgsPtr->arc().filled().color(pfodDwgs::BLACK).radius(5.5).start(ang).angle(0).send();
  if (rh < 30) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6), dwgsPtr->yRadius(ang, 6)).right().fontSize(-7).text(sfLabel.c_str()).send();  
    return;  
  } else if (rh>90) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6), dwgsPtr->yRadius(ang, 6)).left().fontSize(-7).text(sfLabel.c_str()).send();    
    return;
  } // else
  if (rh < 60) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6), dwgsPtr->yRadius(ang, 6)).right().fontSize(-6).text(sfLabel.c_str()).send();
  } else if (rh > 60) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6), dwgsPtr->yRadius(ang, 6)).left().fontSize(-6).text(sfLabel.c_str()).send();
  } else { // == 60
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6), dwgsPtr->yRadius(ang, 6) - 0.15).fontSize(-6).text(sfLabel.c_str()).send();
  }
}

void rh_Gauge::draw() {
  dwgsPtr->arc().filled().color(dwgsPtr->SILVER).radius(5).start(degStart).angle(-degRange).send();
  dwgsPtr->arc().filled().color(dwgsPtr->WHITE).radius(3).idx(z_idx + 1).start(degStart).angle(-degRange).send();
  dwgsPtr->label().fontSize(-3).bold().text(label).color(dwgsPtr->BLACK).offset(0, 5.5).send();
  // labels at 0,20,40,60,80,100
  drawScalePoint(0);
  drawScalePoint(20);
  drawScalePoint(40);
  drawScalePoint(60);
  drawScalePoint(80);
  drawScalePoint(100);
  update(); // update with current state
}

void rh_Gauge::update() {
  dwgsPtr->arc().filled().color(dwgsPtr->RED).radius(5).idx(z_idx).start(degStart).angle(-valueToAngle(value)).send();
  dwgsPtr->label().color(dwgsPtr->RED).offset(0, 3).idx(z_idx + 2).fontSize(1).bold().intValue(value).maxValue(100).displayMax(100).decimals(0).units(F("% RH")).send();
  float ang = degStart - valueToAngle(value);
  dwgsPtr->line().color(dwgsPtr->BLACK).idx(z_idx + 3).size(dwgsPtr->xRadius(ang, 5.5), dwgsPtr->yRadius(ang, 5.5)).send();

}
