/*
   (c)2014-2022 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/

#include "rh_in_out_Gauge.h"
#include "SafeString.h"
// 0 to 100% => 0 to 255 deg
static float valueToAngle(float value) {
  float ang = value * 2.55;
  if (ang > 255) {
    ang = 255;
  }
  if (ang < 0) {
    ang = 0;
  }
  return ang;
}

rh_in_out_Gauge::rh_in_out_Gauge(pfodDwgs *_dwgsPtr) : pfodControl(_dwgsPtr) {
  z_idx = pfodDwgs::reserveIdx(11); // z_idx to z_idx+9
}

void rh_in_out_Gauge::setInRH(int rh) {
  if (rh < 0) {
    rh = 0;
  }
  if (rh > 100) {
    rh = 100;
  }
  rhIn = rh;
}

void rh_in_out_Gauge::setOutRH(int rh) {
  if (rh < 0) {
    rh = 0;
  }
  if (rh > 100) {
    rh = 100;
  }
  rhOut = rh;
}
    
void rh_in_out_Gauge::drawScalePoint(int rh) {
  cSF(sfLabel, 20);
  sfLabel = rh;
  sfLabel += '%';
  float ang = 217.5 - valueToAngle(rh);
  dwgsPtr->arc().filled().color(pfodDwgs::BLACK).radius(6.5).start(ang).angle(0).send();

  if (rh == 0) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6.5), dwgsPtr->yRadius(ang, 6.5)).right().fontSize(-6).text(sfLabel.c_str()).send();
  } else if (rh == 50) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6.5), dwgsPtr->yRadius(ang, 6.5) - 0.6).fontSize(-6).text(sfLabel.c_str()).send();
  } else if (rh == 100) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6.5)+1.5, dwgsPtr->yRadius(ang, 6.5) - 0.15).fontSize(-6).text(sfLabel.c_str()).send();
  } else if (rh < 50) {
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6.5)-0.2, dwgsPtr->yRadius(ang, 6.5)).right().fontSize(-6).text(sfLabel.c_str()).send();
  } else { // (rh > 50) 
    dwgsPtr->label().offset(dwgsPtr->xRadius(ang, 6.5), dwgsPtr->yRadius(ang, 6.5)).left().fontSize(-6).text(sfLabel.c_str()).send();
  }
}

void rh_in_out_Gauge::draw() {
  dwgsPtr->arc().filled().color(dwgsPtr->SILVER).radius(6).start(217.5).angle(-255).send();
  //dwgsPtr->arc().filled().color(dwgsPtr->RED).radius(6).idx(z_idx).start(217.5).angle(-valueToAngle(rhOut)).send(); //in update()
  dwgsPtr->arc().filled().color(dwgsPtr->WHITE).radius(4.5).idx(z_idx + 1).start(217.5).angle(-255).send();
  //dwgsPtr->line().color(dwgsPtr->BLACK).idx(z_idx + 2).size(dwgsPtr->xRadius(angValue, 6), dwgsPtr->yRadius(angValue, 6)).send(); // in update
  dwgsPtr->arc().filled().color(dwgsPtr->SILVER).radius(4).idx(z_idx + 3).start(217.5).angle(-255).send();
  //dwgsPtr->arc().filled().color(dwgsPtr->RED).radius(4).idx(z_idx + 4).start(217.5).angle(-valueToAngle(rhIn)).send(); // in update
  dwgsPtr->arc().filled().color(dwgsPtr->WHITE).radius(2).idx(z_idx + 5).start(217.5).angle(-255).send();
  // dwgsPtr->line().color(dwgsPtr->BLACK).idx(z_idx + 6).size(dwgsPtr->xRadius(angValue, 4), dwgsPtr->yRadius(angValue, 4)).send(); // in update
  dwgsPtr->label().color(dwgsPtr->BLACK).offset(0,2.75).idx(z_idx + 8).bold().fontSize(-1).text("RH").send();

  // labels at 0,25,50,75,100
  drawScalePoint(0);
  drawScalePoint(25);
  drawScalePoint(50);
  drawScalePoint(75);
  drawScalePoint(100);
  
  update(); // update with current state
}

void rh_in_out_Gauge::update() {
//  rhIn = 55;
//  rhOut =78;
  dwgsPtr->arc().filled().color(dwgsPtr->RED).radius(6).idx(z_idx).start(217.5).angle(-valueToAngle(rhOut)).send();
  float angValue = 217.5 - valueToAngle(rhOut);
  dwgsPtr->line().color(dwgsPtr->BLACK).idx(z_idx + 2).size(dwgsPtr->xRadius(angValue, 6), dwgsPtr->yRadius(angValue, 6)).send();
  
  dwgsPtr->arc().filled().color(dwgsPtr->RED).radius(4).idx(z_idx + 4).start(217.5).angle(-valueToAngle(rhIn)).send();
  angValue = 217.5 - valueToAngle(rhIn);
  dwgsPtr->line().color(dwgsPtr->BLACK).idx(z_idx + 6).size(dwgsPtr->xRadius(angValue, 4), dwgsPtr->yRadius(angValue, 4)).send();

  dwgsPtr->label().color(dwgsPtr->RED).offset(0, 6).idx(z_idx + 9).fontSize(0).bold().text(label).intValue(rhIn).maxValue(100).displayMax(100).decimals(0).units(F("% Inside")).send();
  dwgsPtr->label().color(dwgsPtr->RED).offset(0, 9).idx(z_idx + 10).fontSize(0).bold().text(label).intValue(rhOut).maxValue(100).displayMax(100).decimals(0).units(F("% Outside")).send();

}
